/**
 * ANSI/VT terminal escape code handling
 */

export const ANSI_PREFIX = '\x1b[';

/**
 * ANSI escapes for various inputs
 */
export enum KeyCommand {
	ArrowUp = ANSI_PREFIX + 'A',
	ArrowDown = ANSI_PREFIX + 'B',
	ArrowRight = ANSI_PREFIX + 'C',
	ArrowLeft = ANSI_PREFIX + 'D',
	Delete = ANSI_PREFIX + '3~',
	PageUp = ANSI_PREFIX + '5~',
	PageDown = ANSI_PREFIX + '6~',
	Enter = '\r',
	Escape = '\x1b',
	Backspace = '\x7f',

	// These keys have several possible escape sequences
	Home = 'LineHome',
	End = 'LineEnd',
}

export const Ansi = {
	ClearLine: ANSI_PREFIX + 'K',
	ClearScreen: ANSI_PREFIX + '2J',
	ResetCursor: ANSI_PREFIX + 'H',
	HideCursor: ANSI_PREFIX + '?25l',
	ShowCursor: ANSI_PREFIX + '?25h',
	GetCursorLocation: ANSI_PREFIX + '6n',
	InvertColor: ANSI_PREFIX + '7m',
	ResetFormatting: ANSI_PREFIX + 'm',
	moveCursor: (row: number, col: number): string => {
		// Convert to 1-based counting
		row++;
		col++;

		return ANSI_PREFIX + `${row};${col}H`;
	},
	moveCursorForward: (col: number): string => ANSI_PREFIX + `${col}C`,
	moveCursorDown: (row: number): string => ANSI_PREFIX + `${row}B`,
};

export default Ansi;
